<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 09/04/20
 * Time: 11.02
 */

namespace FiloBlu\Rma\Helper;

use FiloBlu\Rma\Api\Data\EasyReturnConfigInterface;
use FiloBlu\Rma\Model\Carrier\MatrixrateFactory;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Model\Quote\Address\RateRequestFactory;
use Magento\Sales\Api\Data\OrderAddressInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;

/**
 *
 */
class EasyReturnHelper extends AbstractHelper
{
    /**
     * @var RateRequestFactory
     */
    private $rateRequestFactory;

    /**
     * @var MatrixrateFactory
     */
    private $matrixRateFactory;

    /**
     * @var DataObjectFactory
     */
    private $objectFactory;

    /**
     * EasyReturnHelper constructor.
     * @param RateRequestFactory $rateRequestFactory
     * @param MatrixrateFactory $matrixRateFactory
     * @param DataObjectFactory $objectFactory
     * @param Context $context
     */
    public function __construct(
        RateRequestFactory $rateRequestFactory,
        MatrixrateFactory  $matrixRateFactory,
        DataObjectFactory  $objectFactory,
        Context            $context)
    {
        parent::__construct($context);
        $this->rateRequestFactory = $rateRequestFactory;
        $this->matrixRateFactory = $matrixRateFactory;
        $this->objectFactory = $objectFactory;
    }


    /**
     * @param OrderAddressInterface $address
     * @param StoreInterface $store
     * @param float $weight
     * @return mixed
     */
    public function simulateEasyReturnShippingCost(OrderAddressInterface $address, StoreInterface $store, float $weight)
    {

        $shipping_amounts [] = 0;
        $request = $this->rateRequestFactory->create();
        $request->setDestCountryId($address->getCountryId());
        $request->setDestRegionId($address->getRegionId());
        $request->setDestPostcode($address->getPostcode());
        $request->setDestCity($address->getCity());
        $request->setPackageWeight($weight);
        $request->setFreeMethodWeight($weight);
        $request->setPackageQty(1);
        $request->setStoreId($store->getId());
        $request->setWebsiteId($store->getWebsiteId());
        $request->setLimitCarrier(null);
        $request->setFreeShipping(null);
        //get rates for the matrixrate request created
        $matrixRate = $this->matrixRateFactory->create();
        $result = $matrixRate->collectRates($request);
        if ($result) {
            $rates = $result->getAllRates();
            if (!empty($rates)) {
                foreach ($rates as $rate) {
                    $price = $rate->getPrice();
                    if (isset($price) && $price > 0) {
                        $shipping_amounts[] = $price;
                    }
                }
            }
        }
        return max($shipping_amounts);
    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isEasyReturnEnabled($storeId = null)
    {
        return $this->scopeConfig->getValue(
            EasyReturnConfigInterface::EASY_RETURN_ENABlE_XML_PATH,
            ScopeInterface::SCOPE_STORE,
            $storeId) ?? false;
    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isForceEasyReturnEnabled($storeId = null)
    {
        return $this->scopeConfig->getValue(
            EasyReturnConfigInterface::FORCE_EASY_RETURN_WHEN_CHANGE_SIZE_ENABlE_XML_PATH,
            ScopeInterface::SCOPE_STORE,
            $storeId) ?? false;
    }

}
